############################################################
#
# pyprelude.file_system
# Copyright (C) 2017, Richard Cook
# Released under MIT License
# https://github.com/rcook/pyprelude
#
############################################################

import os
import shutil
import stat

from .platform import ON_WINDOWS
from .util import unpack_args

def _hack_make_path_doctest_output(s):
    if ON_WINDOWS:
        drive, _ = os.path.splitdrive(os.getcwd())
        assert s.startswith(drive + "\\")
        return s[len(drive) : ].replace("\\", "/")
    else:
        return s

def _remove_dir_on_error(func, path, exc_info):
    if os.access(path, os.W_OK):
        raise
    else:
        os.chmod(path, stat.S_IWUSR)
        func(path)

def remove_dir(path):
    try:
        shutil.rmtree(path, onerror=_remove_dir_on_error)
    except WindowsError:
        with temp_dir() as d:
            status, output, error = execute("robocopy.exe", d, path, "/mir", can_fail=True)
            if status != 2:
                raise RuntimeError("robocopy failed with status {} (output={}, error={})".format(status, output, error))

        os.rmdir(path)

def make_path(*args):
    """
    >>> _hack_make_path_doctest_output(make_path("/a", "b"))
    '/a/b'
    >>> _hack_make_path_doctest_output(make_path(["/a", "b"]))
    '/a/b'
    >>> _hack_make_path_doctest_output(make_path(*["/a", "b"]))
    '/a/b'
    >>> _hack_make_path_doctest_output(make_path("/a"))
    '/a'
    >>> _hack_make_path_doctest_output(make_path(["/a"]))
    '/a'
    >>> _hack_make_path_doctest_output(make_path(*["/a"]))
    '/a'
    """
    paths = unpack_args(*args)
    return os.path.abspath(os.path.join(*[p for p in paths if p is not None]))



############################################################
#
# pyprelude.util
# Copyright (C) 2017, Richard Cook
# Released under MIT License
# https://github.com/rcook/pyprelude
#
############################################################

from __future__ import print_function
import contextlib
import os

def try_pop(d, key, default):
    """
    >>> d = {"a": "b", "c": "d", "e": "f"}
    >>> try_pop(d, "g", "default")
    'default'
    >>> d
    {'a': 'b', 'c': 'd', 'e': 'f'}
    >>> try_pop(d, "c", "default")
    'd'
    >>> d
    {'a': 'b', 'e': 'f'}
    """
    value = d.get(key, default)
    if key in d:
        d.pop(key)
    return value

def unpack_args(*args):
    l = list(args)
    return l[0] if len(l) == 1 and isinstance(l[0], list) else l



############################################################
#
# pyprelude.temp_util
# Copyright (C) 2017, Richard Cook
# Released under MIT License
# https://github.com/rcook/pyprelude
#
############################################################

from __future__ import print_function
import contextlib
import os
import tempfile

from .file_system import remove_dir

@contextlib.contextmanager
def _temp_helper(context_class):
    obj = None
    try:
        obj = context_class()
        yield obj.path
    finally:
        if obj:
            obj.close()

class TempDir(object):
    def __init__(self):
        self._path = tempfile.mkdtemp()

    @property
    def path(self): return self._path

    def close(self):
        if self._path and os.path.isdir(self._path):
            remove_dir(self._path)

class TempFile(object):
    def __init__(self):
        fd, self._path = tempfile.mkstemp()
        os.close(fd)

    @property
    def path(self): return self._path

    def close(self):
        if self._path and os.path.isfile(self._path):
            os.unlink(self._path)

def temp_dir():
    return _temp_helper(TempDir)

def temp_file():
    return _temp_helper(TempFile)

@contextlib.contextmanager
def temp_cwd(dir):
    try:
        saved_dir = os.getcwd()
        os.chdir(dir)
        yield
    finally:
        os.chdir(saved_dir)



############################################################
#
# pyprelude.platform
# Copyright (C) 2017, Richard Cook
# Released under MIT License
# https://github.com/rcook/pyprelude
#
############################################################

import os
import sys

ON_LINUX = sys.platform.startswith("linux")
ON_MACOS = sys.platform.startswith("darwin")
ON_MSYS = (sys.platform.startswith("win") or sys.platform.startswith("msys")) and os.environ.get("SHELL", "").endswith("/bash")
ON_WINDOWS = sys.platform.startswith("win")



############################################################
#
# pyprelude.process
# Copyright (C) 2017, Richard Cook
# Released under MIT License
# https://github.com/rcook/pyprelude
#
############################################################

from __future__ import print_function
import pipes
import subprocess

from .util import try_pop, unpack_args

def _format_command(command):
    return " ".join(map(pipes.quote, map(str, command)))

def proxy_command(name, cwd, build_command, *args, **kwargs):
    command_args = unpack_args(*args)
    can_fail = try_pop(kwargs, "can_fail", False)
    env = try_pop(kwargs, "env", None)
    stdin = try_pop(kwargs, "stdin", None)
    if len(kwargs) > 0:
        raise ValueError("Unsupported keyword argument(s): {}".format(", ".join(kwargs.keys())))

    sanitized_name = name.replace("_", "-")
    command = build_command(sanitized_name, command_args)

    return execute(
        command,
        can_fail=can_fail,
        cwd=cwd,
        stdin=stdin)

def execute(*args, **kwargs):
    command = map(str, unpack_args(*args))
    can_fail = try_pop(kwargs, "can_fail", False)
    cwd = try_pop(kwargs, "cwd", None)
    env = try_pop(kwargs, "env", None)
    stdin = try_pop(kwargs, "stdin", None)
    if len(kwargs) > 0:
        raise ValueError("Unsupported keyword argument(s): {}".format(", ".join(kwargs.keys())))

    p = subprocess.Popen(
        command,
        cwd=cwd,
        env=env,
        stdin=subprocess.PIPE if stdin else None,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE)

    output, error = p.communicate(input=stdin)
    if can_fail:
        return p.returncode, output, error

    if p.returncode != 0:
        raise RuntimeError("Command \"{}\" failed with status {}: {}".format(
            _format_command(command),
            p.returncode,
            error))

    return output



############################################################
#
# pyprelude.url
# Copyright (C) 2017, Richard Cook
# Released under MIT License
# https://github.com/rcook/pyprelude
#
############################################################

import os
import subprocess
import sys
import urllib
import urlparse

from .platform import ON_MACOS, ON_WINDOWS
from .util import unpack_args

def make_url(*args, **kwargs):
    """
    >>> make_url("https://host/api", "users", "user-name", "projects", token="api_token")
    'https://host/api/users/user-name/projects?token=api_token'
    >>> make_url("https://host/api", "users", "user-name", "projects/", token="api_token")
    'https://host/api/users/user-name/projects/?token=api_token'
    >>> make_url("https://host/api", "users", "user-name", "projects", "/", token="api_token")
    'https://host/api/users/user-name/projects/?token=api_token'
    >>> make_url("https://host/api", "users", "user-name", "projects", [("token", "api_token"), ("aaa", "bbb")])
    'https://host/api/users/user-name/projects?token=api_token&aaa=bbb'
    >>> make_url("https://host/api", "users", "user-name", "projects/", [("token", "api_token"), ("aaa", "bbb")])
    'https://host/api/users/user-name/projects/?token=api_token&aaa=bbb'
    >>> make_url("https://host/api", "users", "user-name", "projects", [("token", "api_token"), ("aaa", "bbb")], other="value")
    'https://host/api/users/user-name/projects?token=api_token&aaa=bbb&other=value'
    >>> make_url("https://host/api", "users", "user-name", "projects/", [("token", "api_token"), ("aaa", "bbb")], other="value")
    'https://host/api/users/user-name/projects/?token=api_token&aaa=bbb&other=value'
    >>> make_url("https://host/api")
    'https://host/api'
    >>> make_url("https://host/api/")
    'https://host/api/'
    >>> make_url("https://host/api", "users", "user-name", "projects")
    'https://host/api/users/user-name/projects'
    >>> make_url("https://host/api", "users", "user-name", "projects/")
    'https://host/api/users/user-name/projects/'
    >>> make_url("https://host/api", "users", "user-name", "projects", 12345)
    'https://host/api/users/user-name/projects/12345'
    >>> make_url("https://host/api", "users", "user-name", "projects", 12345, "/")
    'https://host/api/users/user-name/projects/12345/'
    """
    if len(args) == 1 and len(kwargs) == 0:
        # Return a single string unaltered
        return args[0]

    temp_args = unpack_args(*args)
    temp_arg_count = len(temp_args)

    # Handle list of tuples as last of array arguments
    if temp_arg_count > 1 and isinstance(temp_args[-1], list):
        path_fragments = temp_args[0 : -1]
        query_string = urllib.urlencode(temp_args[-1] + kwargs.items())
    elif temp_arg_count > 1 and isinstance(temp_args[-1], dict):
        path_fragments = temp_args[0 : -1]
        query_string = urllib.urlencode(temp_args[-1].items() + kwargs.items())
    else:
        path_fragments = temp_args
        query_string = urllib.urlencode(kwargs)

    path_fragments = map(str, path_fragments)
    has_trailing_slash = path_fragments[-1].endswith("/")
    path_fragments = filter(lambda f: len(f) > 0, map(lambda f: f.strip("/"), path_fragments))
    base_url = "/".join(path_fragments)

    if has_trailing_slash:
        base_url += "/"

    parts = list(urlparse.urlparse(base_url))
    parts[4] = query_string
    return urlparse.urlunparse(parts)

# https://stackoverflow.com/questions/4216985/call-to-operating-system-to-open-url
def open_browser(url):
    if ON_WINDOWS:
        os.startfile(url)
        return True
    elif ON_MACOS:
        subprocess.Popen(["open", url])
        return True
    else:
        try:
            subprocess.Popen(["xdg-open", url])
            return True
        except OSError:
            return False



#!/usr/bin/env python
############################################################
#
# pyprelude
# Copyright (C) 2017, Richard Cook
# Released under MIT License
# https://github.com/rcook/pyprelude
#
############################################################

from __future__ import print_function
from setuptools import setup

setup(
    name="pyprelude",
    version="0.7",
    description="General-purpose Python support functions",
    setup_requires=["setuptools-markdown"],
    long_description_markdown_filename="README.md",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 2.7",
    ],
    url="https://github.com/rcook/pyprelude",
    author="Richard Cook",
    author_email="rcook@rcook.org",
    license="MIT",
    packages=["pyprelude"],
    include_package_data=True,
    test_suite="pyprelude.tests.suite",
    zip_safe=False)
